'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var hoistStatics = _interopDefault(require('hoist-non-react-statics'));

function createStore(reducer, preloadState, enhancer) {
  if (typeof preloadState === 'function' && typeof enhancer === 'undefined') {
    enhancer = preloadState;
    preloadState = undefined;
  }

  if (typeof enhancer !== 'undefined') {
    if (typeof enhancer !== 'function') {
      throw new Error('Expected the enhancer to be a function');
    }

    return enhancer(createStore)(reducer, preloadState);
  }

  var state;
  var listeners = [];

  var getState = function getState() {
    return state;
  };

  var subscribe = function subscribe(listener) {
    listeners.push(listener);
    return function () {
      var index = listeners.indexOf(listener);

      if (index > -1) {
        listeners.splice(index, 1);
      }
    };
  };

  var dispatch = function dispatch(action) {
    state = reducer(state, action);
    listeners.forEach(function (listener) {
      return listener();
    });
    return action;
  };

  dispatch({
    type: '@@redux/INIT'
  });
  return {
    dispatch: dispatch,
    subscribe: subscribe,
    getState: getState
  };
}

var defaultValue = {
  store: undefined,
  parentStore: undefined,
  name: ''
};
var Context = React__default.createContext(defaultValue);

var substores = new Map();
var storeDispatchCovers = new WeakMap();

var Provider = function Provider(_ref) {
  var store = _ref.store,
      name = _ref.name,
      middlewares = _ref.middlewares,
      reducer = _ref.reducer,
      preloadedState = _ref.preloadedState,
      _ref$dispatchBubbles = _ref.dispatchBubbles,
      dispatchBubbles = _ref$dispatchBubbles === void 0 ? true : _ref$dispatchBubbles,
      children = _ref.children;
  var value = React.useMemo(function () {
    // 当前模块卸载时, store不会重新创建, 从map缓存中取, 防止数据丢失
    var currentSubStore = substores.get(name);

    if (!currentSubStore) {
      var substore = createStore(reducer, preloadedState, middlewares);
      substores.set(name, substore);
      currentSubStore = substore;
    }

    var dispatch = currentSubStore.dispatch.bind(currentSubStore); // 防止当前模块内部store dispatch多次赋值, 避免多次调用父级的dispatch方法

    var hasCover = storeDispatchCovers.get(currentSubStore);

    if (!hasCover) {
      currentSubStore.dispatch = function (action) {
        storeDispatchCovers.set(currentSubStore, true);
        dispatch(action);

        if (dispatchBubbles) {
          store.dispatch(action);
        }

        return action;
      };
    }

    return {
      name: name,
      store: currentSubStore,
      parentStore: store
    };
  }, [store, reducer, preloadedState]);
  return React__default.createElement(Context.Provider, {
    value: value
  }, children);
};

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) {
    return;
  }

  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var initStateUpdates = function initStateUpdates() {
  return [null, 0];
};

var storeStateUpdatesReducer = function storeStateUpdatesReducer(state, action) {
  var _state = _slicedToArray(state, 2),
      updateCount = _state[1];

  return [action.payload, updateCount + 1];
};

var defaultMapDispatchToProps = function defaultMapDispatchToProps(dispatch) {
  return {
    dispatch: dispatch
  };
};

var defaultMapStateToProps = function defaultMapStateToProps() {
  return {};
};

function connect(mapStateToProps, mapDispatchToProps) {
  var shouldHandleStateChanges = Boolean(mapStateToProps);

  if (!mapStateToProps) {
    mapStateToProps = defaultMapStateToProps;
  }

  if (!mapDispatchToProps) {
    mapDispatchToProps = defaultMapDispatchToProps;
  }

  return function WrappedWithConnect(WrappedComponent) {
    var wrappedComponentName = WrappedComponent.displayName || WrappedComponent.name || 'Component';
    var displayName = "connect".concat(wrappedComponentName);

    var Connect = function Connect(props) {
      var _useContext = React.useContext(Context),
          store = _useContext.store,
          name = _useContext.name,
          parentStore = _useContext.parentStore;

      var state = _objectSpread2({}, parentStore.getState(), {}, store.getState());

      var stateProps = mapStateToProps(state, props);
      var dispatchProps = mapDispatchToProps(store.dispatch, props); // 分发dispatch时, 强制更新子组件

      var _useReducer = React.useReducer(storeStateUpdatesReducer, null, initStateUpdates),
          _useReducer2 = _slicedToArray(_useReducer, 2),
          forceComponentUpdateDispatch = _useReducer2[1];

      React.useEffect(function () {
        if (!shouldHandleStateChanges) return;

        var forceUpdateFn = function forceUpdateFn() {
          if (substores.get(name) !== store) return;
          forceComponentUpdateDispatch({
            type: '@@redux/STORE_UPDATED'
          });
        };

        var unsubscribe = store.subscribe(forceUpdateFn);
        return function () {
          unsubscribe();
        };
      }, [store, parentStore]);
      return React__default.createElement(WrappedComponent, Object.assign({}, props, stateProps, dispatchProps));
    };

    Connect.WrappedComponent = WrappedComponent;
    Connect.displayName = displayName;
    return hoistStatics(Connect, WrappedComponent);
  };
}

exports.Provider = Provider;
exports.connect = connect;
exports.createStore = createStore;
