import {
  Reducer,
  StoreEnhancer,
  AnyAction,
  Dispatch,
} from 'redux';
import { Store, AnyFunction } from './types';

function createStore(
  reducer: Reducer<any>,
  preloadState?: any,
  enhancer?: any,
): Store {
  if (
    typeof preloadState === 'function'
    && typeof enhancer === 'undefined'
  ) {
    enhancer = preloadState as (StoreEnhancer<any> | undefined);
    preloadState = undefined;
  }

  if (typeof enhancer !== 'undefined') {
    if (typeof enhancer !== 'function') {
      throw new Error('Expected the enhancer to be a function');
    }

    return enhancer(createStore)(reducer, preloadState);
  }

  let state: any;
  let listeners: AnyFunction[] = [];

  const getState = () => state;

  const subscribe = (listener: AnyFunction) => {
    listeners.push(listener);

    return () => {
      const index = listeners.indexOf(listener);

      if (index > -1) {
        listeners.splice(index, 1);
      }
    };
  };

  const dispatch: Dispatch<AnyAction> = action => {
    state = reducer(state, action);

    listeners.forEach(listener => listener());

    return action;
  };

  dispatch({ type: '@@redux/INIT' });

  return {
    dispatch,
    subscribe,
    getState
  };
}

export default createStore;
